import torch
from torch import Tensor
if torch.cuda.is_available():
    from quad_iou_cuda import calculateIoUCuda
from quad_iou_cpu import calculateIoUCPU


def calculate_iou(quad_0: Tensor, quad_1: Tensor, sort_input_quads: bool = True) -> Tensor:
    """
    Calculates the Intersection over Union (IoU) between two sets of quadrilaterals.

    Parameters:
    - quad_0 (Tensor): A tensor of shape (M, 4, 2) representing the coordinates of M quadrilaterals. Each quadrilateral
      is defined by 4 vertices, and each vertex has 2 coordinates (x, y).
    - quad_1 (Tensor): A tensor of shape (N, 4, 2) representing the coordinates of N quadrilaterals. Each quadrilateral
      is defined similarly to those in quad_0.
    - sort_input_quads (bool, optional): Whether to sort the vertices of the quadrilaterals before calculating IoU.
      Sorting can be necessary depending on the specific calculation requirements. Default is True.

    Returns:
    - Tensor: An (N, M) tensor where each element [i, j] is the IoU between the i-th quadrilateral from quad_1 and the
      j-th quadrilateral from quad_0.

    This function computes the IoU for each pair of quadrilaterals from the two input tensors and returns a matrix
    of these IoU values.
    """
    if not isinstance(quad_0, Tensor): raise ValueError(f"Expected input is Tensor but got {type(quad_0)}")
    if not isinstance(quad_1, Tensor): raise ValueError(f"Expected input is a Tensor but got {type(quad_1)}")
    if not isinstance(sort_input_quads, bool): raise ValueError(f"Expected sort_input_quads to be a boolean but got {type(sort_input_quads)}")
    if quad_0.device == quad_1.device:
        if quad_0.device.type == "cpu":
            return calculateIoUCPU(quad_0, quad_1, sort_input_quads)
        elif quad_0.device.type == "cuda":
            return calculateIoUCuda(quad_0, quad_1, sort_input_quads)
    raise ValueError(f"Expected all tensors to be on the same device but got device {quad_0.device} for quad_0 and {quad_1.device} for quad_1")



from setuptools import setup, find_packages
from torch.utils.cpp_extension import BuildExtension, CUDAExtension, CppExtension
import torch

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

ext_modules = [
    CppExtension("quad_iou_cpu", [
        "src/core/quad_iou_tensors_bind_cpu.cpp",
        "src/core/quad_iou_tensors.cpp",
    ],
    extra_compile_args=['-fopenmp'],
    extra_link_args=['-fopenmp']
    )
]

if torch.cuda.is_available():
    ext_modules.append(
        CUDAExtension('quad_iou_cuda', [
            'src/core/quad_iou_tensors_bind_cuda.cpp',
            'src/core/quad_iou_tensors.cu',
        ])
    )

setup(
    name="quad_iou",
    version="0.2.0",
    author="Irakli Salia",
    author_email="irakli.salia854@gmail.com",
    description="Torch extension for calculating IoU(Intersection over Union) for quadrilaterals(MxN)",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/Isalia20/quad-iou-torch",
    package_dir={'': 'src'},
    packages=find_packages(where="src"),
    ext_modules=ext_modules,
    cmdclass={
        'build_ext': BuildExtension
    },
    python_requires='>=3.9',
)